﻿using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LordAshes
{
    public partial class DnD5eLinkData : ILinkData
    {
        private Dictionary<string, object> stats = new Dictionary<string, object>();

        private string dataFolder = "";
        private string name = "";

        private bool changed = false;

        public void ProcessData(Dictionary<string, dynamic> data, string rootDataFolder)
        {
            this.dataFolder = rootDataFolder;
            changed = false;

            name = (data.ContainsKey("Name") ? data["Name"] : "Unknown");

            Console.WriteLine("[ " + name + " -> Dnd5eLinkData ]");

            foreach (KeyValuePair<string, dynamic> item in data)
            {
                ProcessElement(data["Name"], item);
            }
            if(changed)
            {
                Console.WriteLine("Updating Rule Set 5e Character File '" + data["Name"] + ".DnD5e" + "'");
                System.IO.File.WriteAllText(dataFolder + data["Name"] + ".DnD5e", "");
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "{\r\n");
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t\"NPC\": false,\r\n");
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t\"reach\": 5,\r\n");
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t\"attacks\":\r\n");
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t[\r\n");
                for(int i=0; i<Templates.sections["attacks"].Count; i++)
                {
                    System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", Templates.sections["attacks"].ElementAt(i));
                    if (i < Templates.sections["attacks"].Count - 1)
                    {
                        System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t\t,\r\n");
                    }
                }
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t],\r\n");
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t\"saves\":\r\n");
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t[\r\n");
                for (int i = 0; i < Templates.sections["saves"].Count; i++)
                {
                    System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", Templates.sections["saves"].ElementAt(i));
                    if (i < Templates.sections["saves"].Count - 1)
                    {
                        System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t\t,\r\n");
                    }
                }
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t],\r\n");
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t\"skills\":\r\n");
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t[\r\n");
                for (int i = 0; i < Templates.sections["skills"].Count; i++)
                {
                    System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", Templates.sections["skills"].ElementAt(i));
                    if (i < Templates.sections["skills"].Count - 1)
                    {
                        System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t\t,\r\n");
                    }
                }
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t],\r\n");
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t\"immunity\": [],\r\n");
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "\t\"resistance\": []\r\n");
                System.IO.File.AppendAllText(dataFolder + data["Name"] + ".DnD5e", "}\r\n");
            }
        }

        public void ProcessElement(string name, KeyValuePair<string, dynamic> element, string root = "")
        {
            // Resurse through the hierarchy
            if (element.Value.GetType().ToString().EndsWith("JObject"))
            {
                // JObjects are sub-level dictionaries
                foreach (KeyValuePair<string, dynamic> item in JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(element.Value.ToString()))
                {
                    ProcessElement(name, item, (root == "") ? element.Key : root + "." + element.Key);
                }
            }
            else
            {
                bool requiresUpdate = true;
                if (element.Value.ToString() != "")
                {
                    string key = name + "." + ((root == "") ? element.Key : root + "." + element.Key);
                    if (stats.ContainsKey(key))
                    {
                        // Old stat - check for change
                        if (stats[key].ToString() == element.Value.ToString())
                        {
                            // No change - avoid update
                            requiresUpdate = false;
                        }
                        else
                        {
                            // Change - remove key (the updated key will be added below)
                            Console.WriteLine(key + " : Changed To " + element.Value.ToString());
                            stats.Remove(key);
                        }
                    }
                    else
                    {
                        // New entry
                        Console.WriteLine(key + " : New With " + element.Value.ToString());
                    }
                    if (requiresUpdate)
                    {
                        changed = true;
                        // Change to any stat which is used for rolls causes DSM (DiceSelectionMacro) file update
                        stats.Add(key, element.Value);
                        if (key.StartsWith(name + ".Saves.") ||
                           key.StartsWith(name + ".Init") ||
                           key.StartsWith(name + ".Attacks.") ||
                           key.StartsWith(name + ".Skills.") ||
                           key.StartsWith(name + ".Abilities.") ||
                           key.StartsWith(name + ".Prof"))
                        {
                            Templates.sections.Clear();
                            Templates.sections.Add("attacks", new List<string>());
                            Templates.sections.Add("saves", new List<string>());
                            Templates.sections.Add("skills", new List<string>());
                            foreach (KeyValuePair<string, object> stat in stats)
                            {
                                if (stat.Key.StartsWith(name + ".Init"))
                                {
                                    Templates.sections["attacks"].Add(MakeSkillRoll("Initiative","Public","1D20"+ stat.Value.ToString()));
                                }
                            }
                            foreach (KeyValuePair<string, object> stat in stats)
                            {
                                if (stat.Key.StartsWith(name + ".Attacks.") && stat.Key.Contains("damage.amount"))
                                {
                                    string rootKey = stat.Key.Replace(".damage.amount", "");
                                    Templates.sections["attacks"].Add(MakeAttackRoll(rootKey.Substring(rootKey.LastIndexOf(".") + 1), stats[rootKey + ".type"].ToString(), "1D20" + stats[rootKey], stats[rootKey+".range"].ToString(), stats[rootKey + ".type"].ToString(), stats[rootKey + ".damage.type"].ToString(), stats[rootKey + ".damage.amount"].ToString()));
                                }
                            }
                            foreach (KeyValuePair<string, object> stat in stats)
                            {
                                if (stat.Key.StartsWith(name + ".Saves."))
                                {
                                    Templates.sections["saves"].Add(MakeSkillRoll(stat.Key.Substring(stat.Key.LastIndexOf(".") + 1), "Public", "1D20" + stat.Value.ToString()));
                                }
                            }
                            foreach (KeyValuePair<string, object> stat in stats)
                            {
                                if (stat.Key.StartsWith(name + ".Skills."))
                                {
                                    Templates.sections["skills"].Add(MakeSkillRoll(stat.Key.Substring(stat.Key.LastIndexOf(".") + 1), "Private", "1D20" + stat.Value.ToString()));
                                }
                            }
                            foreach (KeyValuePair<string, object> stat in stats)
                            {
                                if (stat.Key.StartsWith(name + ".Abilities."))
                                {
                                    Templates.sections["skills"].Add(MakeSkillRoll(stat.Key.Substring(stat.Key.LastIndexOf(".") + 1), "Private", "1D20" + stat.Value.ToString()));
                                }
                            }
                            foreach (KeyValuePair<string, object> stat in stats)
                            {
                                if (stat.Key.StartsWith(name + ".Prof"))
                                {
                                    Templates.sections["skills"].Add(MakeSkillRoll(stat.Key.Substring(stat.Key.LastIndexOf(".") + 1), "Private", "1D20" + stat.Value.ToString()));
                                }
                            }
                        }
                        else
                        {
                            // Other stats are created as individual files
                            System.IO.File.WriteAllText(dataFolder + key, element.Value.ToString());
                        }
                    }
                }
            }
        }
    }
}
